@@ -2,4 +2,4 @@
 
 name="fastspell"
 
-from . import fastspell
\ No newline at end of file
+from .fastspell import FastSpell
