@@ -107,38 +107,59 @@ class FastSpell:
             urllib.request.urlretrieve("https://dl.fbaipublicfiles.com/fasttext/supervised-models/lid.176.bin", ft_model_path)
             self.model = fasttext.load_model(ft_model_path) 
 
-        self.similar = self.similar_langs.get(lang)
+        # Obtain all the possible lists for the given lang
+        # a.k.a the list for each script of the lang
+        self.similar = []
+        for sim_entry in self.similar_langs:
+            if sim_entry.split('_')[0] == lang:
+                self.similar.append(self.similar_langs[sim_entry])
 
         #If there are languages that can be mistaken 
         #with the target language: prepare an array of Hunspell spellcheckers
         #for all the similar languages
-        if self.similar != None:            
-            for l in self.similar:
+        logging.debug(f"Similar lists for '{lang}': {self.similar}")
+        for similar_list in self.similar:
+            for l in similar_list:
+                if l in self.hunspell_objs:
+                    continue # Avoid loading one dic twice
                 #load dicts
                 try:
-                    dict = self.dictpath+self.hunspell_codes.get(l)
+                    dict = self.dictpath + '/' + self.hunspell_codes.get(l)
                     hunspell_obj = hunspell.HunSpell(dict+'.dic', dict+'.aff') 
                     self.hunspell_objs[l] = hunspell_obj
+                    logging.debug(f"Loaded hunspell obj for '{l}' in path: {dict}")
                 except hunspell.HunSpellError:
                     logging.error("Failed building Hunspell object for "+l)
                     logging.error("Please check that " + dict+".dic" + " and " + dict+'.aff' + " do exist.")
                     logging.error("Aborting.") 
                     exit(1)
+        # The code above will load for this similar.yaml:
+        # hbs_lat: [hbs_lat, sl]
+        # hbs_cyr: [hbs_cyr, ru, mk, bg]
+        # the subsequent list of hunspell objs:
+        # hunspell_objs = [hbs_lat, sl, hbs_cyr, ru, mk, bg]
 
         # Crate translate tables for script detection
-        if script:
-            self.script_tables = {
-                'hbs_lat': str.maketrans('', '', 'aAbBcčČćĆdDđĐeEfFgGhHiIjJkKlLmMnNoOpPrRsSšŠtuUvVzZžŽ'),
-                'hbs_cyr': str.maketrans('', '', 'АаБбВвГгДддЂђЕеЖжЗзИиЙйКкkЛлЉљМмНнЊњОоПпРрЋћТтУуФфХхЦцЧчШшЩщ'),
-            }
+        self.script_tables = {
+            "hbs": {
+                # Combination of Gaj's alphabet and Montenegrin Latin
+                # plus unicode chars of double letters
+                'hbs_lat': str.maketrans('', '',
+                    'aAbBcčČćĆdDđĐeEfFgGhHiIjJkKlLmMnNoOpPrRsSšŠŚśtuUvVzZžŽŹźﬁﬂﬆĳœǌ'),
+                # Combination of Serbian Cyrillic and Montenegrin Cyrillic
+                'hbs_cyr': str.maketrans('', '',
+                    'АаБбВвГгДддЂђЕеЖжЗзЗ́з́ИиКкkЛлЉљМмНнЊњОоПпРрСсС́с́ЋћТтУуФфХхЦцЧчШшЩщҵҥӕ'),
+            },
+        }
 
-    def getscript(self, sent):
+    def getscript(self, sent, lang):
         # Return as detected script the one that its translate table
         # deletes more characters
+        script_tables = self.script_tables[lang] # grab script tables of the requested lang
         best_count = sys.maxsize
         best_script = None
-        for script in self.script_tables.keys():
-            count_chars = len(sent.translate(self.script_tables[script]))
+        for script in script_tables.keys():
+            count_chars = len(sent.translate(script_tables[script]))
             if count_chars < best_count:
                 best_count = count_chars
                 best_script = script
@@ -147,33 +168,42 @@ class FastSpell:
 
 
     def getlang(self, sent):
-        
         sent=sent.replace("\n", " ").strip()
-        prediction = self.model.predict(sent, k=1)[0][0][len(self.prefix):]
+        prediction = self.model.predict(sent.lower(), k=1)[0][0][len(self.prefix):]
 
         # Return 'hbs' for all serbo-croatian variants
         # if hbs mode is enabled or hbs is the requested language
         if (self.hbs or self.lang == 'hbs') and prediction in HBS_LANGS:
-            if self.script:
-                return self.getscript(sent)
-            return 'hbs'
+            prediction = 'hbs'
 
-        #classic norwegian ñapa
+        # If prediction does not specify the with variant
+        # replace it by any of the variants to trigger hunspell refinement
         if prediction == "no":
             prediction = "nb"
-        #New serbo-croatian ñapa    
         if prediction == "sh":
             prediction = "sr"
-            
+
+        # Always detect script if supported (will be printed only if requested)
+        script = ''
+        if prediction in self.script_tables:
+            prediction = self.getscript(sent, prediction)
+            logging.debug(f"Detected script {prediction}")
+
         #TODO: Confidence score?
 
-        if self.similar == None or prediction not in self.similar:
+        if self.similar == [] or prediction not in self.hunspell_objs:
         #Non mistakeable language: just return FastText prediction
-            return(prediction)
+            refined_prediction = prediction
         else:
         #The target language is mistakeable
+            # Obtain the list of languages to spellcheck, only similar for the current lang and script
+            current_similar = None
+            for sim_list in self.similar:
+                if prediction in sim_list or f'{prediction}_{script}' in sim_list:
+                    current_similar = sim_list
+
             spellchecked = {}
-            for l in self.hunspell_objs:
+            for l in current_similar:
                 #Get spellchecking for all the mistakeable languages
                 logging.debug(l)
                 dec_sent = sent.encode(encoding='UTF-8',errors='strict').decode('UTF-8') #Not 100% sure about this...
@@ -196,6 +226,7 @@ class FastSpell:
                     spellchecked[l] =  error_rate
                 logging.debug("----------------")
 
+            logging.debug(f"Spellchecked: {spellchecked}")
             if len(spellchecked) > 0:
                 #at least one of the spellchecks was below the threshold            
                 #get best values and keys
@@ -203,28 +234,36 @@ class FastSpell:
                 best_keys = [k for k, v in spellchecked.items() if v == best_value]
                 if len(best_keys)==1:
                     #Only one language scoring the best
-                    return(best_keys[0])
+                    refined_prediction = best_keys[0]
                 else:
                     #It's a tie!
                     if self.mode == "aggr":
                         #Aggressive approach: if the targetted language is among the best scoring, take it
                         if self.lang in best_keys:
-                            return(self.lang)
+                            refined_prediction = self.lang
                         elif prediction in best_keys:
                             #the targetted language is not in the best ones, and the prediction?
-                            return(prediction)
+                            refined_prediction = prediction
                         else:
                             #Just take one
-                            return(best_keys[0])
+                            refined_prediction = best_keys[0]
                     if self.mode == "cons":
                         #Conservative: just keep it as unknown
-                        return("unk")
+                        refined_prediction = "unk"
             else:
                 #Nothing in the spellchecking list
                 if self.mode == "aggr":
-                    return(prediction)
+                    refined_prediction = prediction
                 else:
-                    return("unk")
+                    refined_prediction = "unk"
+
+        # If script detection not requested
+        # remove it from prediction
+        if self.script:
+            return refined_prediction
+        else:
+            return refined_prediction.split('_')[0]
+
 
 def perform_identification(args):
     time_start = timeit.default_timer()
